//----------------------------------------------------------------
// "A Game of Intelligent Risk" by David Russell Newman (drn101) 
// Class: Box
// Version: 1.0
// Last Modified: 14/02/2003
//----------------------------------------------------------------

/** 
 * This class creates the a Box Object which is part of the game
 * Grid and can hold both  numbers of blue and red men. 
 * @see java.lang.Object
 */
public class Box{
	public static final int EMPTY = 0;
	public static final int RED = 1;
	public static final int BLUE = 2;
	public static final int BOTH = 3;
	int posSize[]; 
	boolean sel;
	int col;
	long blueNum, redNum;
	BoxPosition bp;

	/**
	* Constructs a new Box Object when passed the location of the Box in both
	* terms of the display panel and the Grid.
	* @param x the horizontal position of the top left corner of the Box on the 
	* display panel.
	* @param y the vertical position of the top left corner of the Box on the 
	* display panel.
	* @param xLen the horizontal length of the Box.
	* @param yLen the vertical length of the Box.
	* @param horNumPar the horizontal position of the Box in terms of the Grid.
	* @param vertNumPar the vertical position of the Box in terms of the Grid.
	*/
	public Box(int x, int y, int xLen, int yLen, int horNumPar, int vertNumPar){
		posSize = new int[4];
		posSize[0] = x;
		posSize[1] = y;
		posSize[2] = xLen;
		posSize[3] = yLen;
		bp = new BoxPosition(horNumPar, vertNumPar);
		sel = false;
		col = 0;
		blueNum = 0;
		redNum = 0;
	}

	/**
	* Returns a array of ints that contains the position and size of the Box.
	* @returns an int[] that contains the position and size of the Box.
	*/
	public int[] getPosSize(){
		return posSize;
	}

	/**
	* Sets the colour of the Box (in terms of an int) to the passed parameter.
	* Box.EMPTY = 0, Box.RED = 1, Box.BLUE = 2, Box.BOTH = 3.
	* @param colPar an int representing the colour to set the Box to.
	*/ 
	public void setCol(int colPar){
		col = colPar;
	}

	/**
	* Gets the colour of the Box, in terms of an int.
	* Box.EMPTY = 0, Box.RED = 1, Box.BLUE = 2, Box.BOTH = 3.
	* @returns an int representing the colour of the Box.
	*/
	public int getCol(){
		return col;
	}

	/**
	* Returns the number of men in the Box of the colour passed as a parameter.
	* @param col the colour (as an int) of the side for which the number of men
	* of that colour will be returned.
	* @returns a long representing the number of men in the Box of the colour 
	* specified by the parameter.
	*/
	public long getNum(int col){
		if (col==BLUE) return blueNum;
		else if (col==RED) return redNum;
		else return 0;
	}

	/**
	* Sets the the number of men of the colour of the first parameter to that 
	* of the second parameter.
	* @param colPar the colour (as an int) for which this method is to set men 
	* for.
	* @param numPar the number of men to set the appropriate colour's number of 
	* of men to.
	*/
	public void setNum(int colPar, int numPar){
		if (colPar==BLUE) blueNum = numPar;
		else if (colPar==RED) redNum = numPar;

	}

	/**
	* Returns the number of blue men in this Box.
	* @returns the number of blue men in this Box.
	*/
	public long getBlueNum(){
		return blueNum;
	}

	/**
	* Sets the number of blue men of this Box to the passed parameter.
	* @param blueNumPar the number to set the number of blue men of this Box to.
	*/
	public void setBlueNum(long blueNumPar){
		blueNum = blueNumPar;
	}

	/**
	* Returns the number of red men in this Box.
	* @returns the number of red men in this Box.
	*/
	public long getRedNum(){
		return redNum;
	}
	
	/**
	* Sets the number of red men of this Box to the passed parameter.
	* @param redNumPar the number to set the number of red men of this 
	* Box to.
	*/
	public void setRedNum(long redNumPar){
		redNum = redNumPar;
	}

	/**
	* Returns the horizontal position in terms of the Grid.
	* @returns the horizontal position in terms of the Grid.
	*/
	public int getHorNum(){
		return bp.getHoriz();
	}

	/**
	* Returns the vertical position in terms of the Grid.
	* @returns the vertical position in terms of the Grid.
	*/
	public int getVertNum(){
		return bp.getVert();
	}
	
	/**
	* Returns the BoxPosition, (an Object holding both the horizontal and 
	* vertical position of the Box in trems of the Grid), of the Box. 
	* @returns the BoxPosition, (an Object holding both the horizontal and 
	* vertical position of the Box in trems of the Grid), of the Box. 
	*/
	public BoxPosition getBoxPosition(){
		return bp;
	}
}			
	
	 