//----------------------------------------------------------------
// "A Game of Intelligent Risk" by David Russell Newman (drn101) 
// Class: RiskGame
// Version: 1.0
// Last Modified: 17/12/2003
//----------------------------------------------------------------

import java.util.*;
import java.io.*;
import javax.swing.*;

public class RiskGame{
	public static final int RANDOM = 1, ONE_MAN = 1, BATTLE = 2, CHOSEN = 2; 
	public static final int NONE = 3, SQUARE_SELECT = 3, FROM_FILE = 4, DEFENDER_WINS = 1;
	public static final int ATTACKER_WINS = 2, REROLLED = 3, LAST_PLAYER = 1, SCORED = 2; 
	public static final int INITIALISATION = 1, SQUARE_SELECTING = 2, MAN_SELECTING = 3, IN_GAME = 4;
	private int gridX, gridY, noMen, noInitSquares, noTurns, manPoints, squarePoints, whoseTurn, gameState;
	private int squareAlloc, manAlloc, scoreResult, scoreMaxInt, winner, movesPerTurn, draws, turnsTaken;
	private boolean defenderWd, attackerWd, diagonalSquaresAdjacent, weighted, scoreMaxConstant, battleInProgress;
	private String menAddedFunc, scoreMaxVariable, curStr;
	public Vector tokens;
	private int y=0;
	private int z=0;
	private int lastWinner=0;
	private long lastRedScore = 0;
	private long lastBlueScore = 0;
	private Grid theGrid;
	private Player bluePlayer, redPlayer;
	private BoxPosition fromBox, toBox, curBox, battleBox;
	private boolean defWD, attWD;
	private int movesLeft;
	private boolean finishedGame = false;

	public RiskGame(){
		gridX = 0;
		gridY = 0;
		noMen = 0;
		noInitSquares = 0;
		noTurns = 0;
		manPoints = 0;
		squarePoints = 0;
		squareAlloc = 0;
		manAlloc = 0;
		scoreResult = 0;
		scoreMaxInt = 0;
		winner = 0;
		movesPerTurn = 0;
		draws = 0;
		menAddedFunc = new String();
		scoreMaxVariable = new String();
		curStr = new String();
		tokens = new Vector();
		whoseTurn=Box.RED;
		gameState=INITIALISATION;
		fromBox=null;
		toBox=null;
		curBox=null;
		battleBox=null;
		battleInProgress = false;
		defWD = false;
		attWD = false;
	}
	public void addMenToSquares(){
		for (int i=0; i<gridX; i++){
			for (int j=0; j<gridY; j++){
			//	System.out.println("X = "+i+", Y = "+j);
				if (getGrid().getBox(i,j).getRedNum()>0){
		//			System.out.println("Red Num = "+getGrid().getBox(i,j).getRedNum()); 
					double res = calcNewNumMen(getGrid().getBox(i,j).getRedNum(), new Vector(tokens));
					long resRound = Math.round(res);
					if (resRound-res>0) resRound = resRound-1;
					CustomPanelTest.addMenGained(Box.RED, resRound-getGrid().getBox(i,j).getRedNum());
					getGrid().getBox(i,j).setRedNum(resRound);
			//		System.out.println("Red Num Set"); 
				}
				if (getGrid().getBox(i,j).getBlueNum()>0){
			//		System.out.println("Blue Num = "+getGrid().getBox(i,j).getBlueNum()); 
					double res = calcNewNumMen(getGrid().getBox(i,j).getBlueNum(), new Vector(tokens));
					long resRound = Math.round(res);
					if (resRound-res>0) resRound = resRound-1;
					CustomPanelTest.addMenGained(Box.BLUE, resRound-getGrid().getBox(i,j).getBlueNum());
					getGrid().getBox(i,j).setBlueNum(resRound);	
			//		System.out.println("Blue Num Set"); 
				}		
			}
		}
	}		
	public void decrementMovesLeft(){
		movesLeft--;
	}
	public int getMovesLeft(){
		return movesLeft;
	}
	public void resetMovesLeft(){
		movesLeft = movesPerTurn;
	}
	public boolean getDefWDing(){
		return defWD;
	}
	public void setDefWDing(boolean defWDPar){
		defWD = defWDPar;
	}
	public boolean getAttWDing(){
		return attWD;
	}
	public void setAttWDing(boolean attWDPar){
		attWD = attWDPar;
	}
	public void tidyBox(BoxPosition bpPar){
		Box theBox = theGrid.getBox(bpPar);
		if (theBox.getRedNum()>0){
			if (theBox.getBlueNum()>0)
				theBox.setCol(Box.BOTH);
			else 
				theBox.setCol(Box.RED);
		}
		else{
			if (theBox.getBlueNum()>0)
				theBox.setCol(Box.BLUE);
			else 
				theBox.setCol(Box.EMPTY);
		}
	}
	public void setLastWinner(int lastWinnerPar){
		lastWinner = lastWinnerPar;
	}
	public int getLastWinner(){
		return lastWinner;
	}
	public void setLastRedScore(long lastRSPar){
		lastRedScore = lastRSPar;
	}
	public long getLastRedScore(){
		return lastRedScore;
	}
	public void setLastBlueScore(long lastBSPar){
		lastBlueScore = lastBSPar;
	}
	public long getLastBlueScore(){
		return lastBlueScore;
	}
	public void setBattleBox(BoxPosition battleBoxPar){
		battleBox = battleBoxPar;
	}
	public BoxPosition getBattleBox(){
		return battleBox;
	}
	public void setBattleInProgress(boolean bipPar){
		battleInProgress = bipPar;
	}
	public boolean getBattleInProgress(){
		return battleInProgress;
	}
	public void setFromBox(BoxPosition fromBoxPar){
		fromBox = fromBoxPar;
	}
	public BoxPosition getFromBox(){
		return fromBox;
	}
	public void setToBox(BoxPosition toBoxPar){
		toBox = toBoxPar;
	}
	public BoxPosition getToBox(){
		return toBox;
	}
	public void setCurBox(BoxPosition curBoxPar){
		curBox = curBoxPar;
	}
	public BoxPosition getCurBox(){
		return curBox;
	}		
	public int getGameState(){
		return gameState;
	}
	public void setGridX(int gridXPar){
		gridX = gridXPar;
	}
	public int getGridX(){
		return gridX;
	}
	public void setGridY(int gridYPar){
		gridY = gridYPar;
	}
	public int getGridY(){
		return gridY;
	}
	public void setGrid(){
		theGrid = new Grid(gridX, gridY);
	}
	public Grid getGrid(){
		return theGrid;
	}
	public void setNoMen(int noMenPar){
		noMen = noMenPar;
	}
	public int getNoMen(){
		return noMen;
	}
	public void setNoInitSquares(int noInitSquaresPar){
		noInitSquares = noInitSquaresPar;
		redPlayer = new Player(Box.RED, noMen, noInitSquares);
		bluePlayer = new Player(Box.BLUE, noMen, noInitSquares);
	}
	public int getNoInitSquares(){
		return noInitSquares;
	}
	public Player getPlayer(int col){
		if (col==Box.RED) return redPlayer;
		else if (col==Box.BLUE) return bluePlayer;
		else return null;
	}
	public Player getRedPlayer(){
		return redPlayer;
	}
	public Player getBluePlayer(){
		return bluePlayer;
	}
	public void setNoTurns(int noTurnsPar){
		noTurns = noTurnsPar;
	}
	public int getNoTurns(){
		return noTurns;
	}
	public void setPointsPerMan(int manPointsPar){
		manPoints = manPointsPar;
	}
	public int getPointsPerMan(){
		return manPoints;
	}
	public void setPointsPerSquare(int squarePointsPar){
		squarePoints = squarePointsPar;
	}
	public int getPointsPerSquare(){
		return squarePoints;
	}
	public void setSquareAllocStatus(int squareAllocPar){
		squareAlloc = squareAllocPar;
	}
	public int getSquareAllocStatus(){
		return squareAlloc;
	}
	public void setManAllocStatus(int manAllocPar){
		manAlloc = manAllocPar;
	}
	public int getManAllocStatus(){
		return manAlloc;
	}
	public void setScoreResultStatus(int scoreResultPar){
		scoreResult = scoreResultPar;
	}
	public int getScoreResultStatus(){	
		return scoreResult;
	}
	public void setScoreMaxInt(int scoreMaxPar){
		scoreMaxInt = scoreMaxPar;
		scoreMaxConstant = true;
	}
	public int getScoreMaxInt(){
		return scoreMaxInt;
	}
	public String getScoreMaxVariable(){
		return scoreMaxVariable;
	}
	public void setScoreMaxVariable(String scoreMaxPar){
		scoreMaxVariable = scoreMaxPar;
		scoreMaxConstant = false;
	}
	public long getScoreMax(Box battleBox){
		if (!scoreMaxConstant){
			if (scoreMaxVariable.equals("NO_SQUARES")){
				return gridX*gridY;
			}
			else if (scoreMaxVariable.equals("NO_OF_MEN_IN_BATTLE")){
				return battleBox.getBlueNum()+battleBox.getRedNum();
			}
		}
		return scoreMaxInt;
	}
	public void setWinnerStatus(int winnerPar){
		winner = winnerPar;
	}
	public int getWinnerStatus(){
		return winner;
	} 
	public void setMovesPerTurn(int movesPerTurnPar){
		movesPerTurn = movesPerTurnPar;
		movesLeft = movesPerTurn;
	}
	public int getMovesPerTurn(){
		return movesPerTurn;
	}
	public void setDefenderWithdraw(boolean defenderWdPar){
		defenderWd = defenderWdPar;
	}
	public boolean getDefenderWithdraw(){
		return defenderWd;
	}
	public void setAttackerWithdraw(boolean attackerWdPar){
		attackerWd = attackerWdPar;
	}
	public boolean getAttackerWithdraw(){
		return attackerWd;
	}
	public void setDiagonalSquaresAdjacent(boolean diagSqAdjPar){
		diagonalSquaresAdjacent = diagSqAdjPar;
	}
	public boolean getDiagonalSquaresAdjacent(){
		return diagonalSquaresAdjacent;
	}
	public void setWeighted(boolean weightedPar){
		weighted = weightedPar;
	}
	public boolean getWeighted(){
		return weighted;
	}
	public void setMenAddedFunction(String menAddedFuncPar){
		if (parseAddMenFunc(menAddedFuncPar))
			menAddedFunc = menAddedFuncPar;
	}
	public String getMenAddedFunction(){
		return menAddedFunc;
	}
	public void setDraws(int drawsPar){
		draws = drawsPar;
	}
	public int getDraws(){
		return draws;
	}
	public int getTurn(){
		return whoseTurn;
	}
	public void swapTurns(){
		if (whoseTurn==Box.RED){
			whoseTurn = Box.BLUE;
		}
		else if (whoseTurn==Box.BLUE){
			whoseTurn = Box.RED;
			if (gameState==RiskGame.IN_GAME){
				turnsTaken++; 
				if (noTurns!=turnsTaken)
					addMenToSquares();
			}
			if (noTurns-turnsTaken==0 && noTurns!=0)
				finishedGame=true;
		}
	}
	public void setGameFinished(){
		finishedGame=true;
	}
	public boolean getGameFinished(){
		return finishedGame;
	}
	public long getMen(int colPar){
		if (colPar==Box.RED) return getRedMen();
		else return getBlueMen();
	}
	public long getRedMen(){
		long redMenNum = 0;
		Enumeration enum1 = redPlayer.getOccupiedSquares().elements();
		while (enum1.hasMoreElements()){
			redMenNum =  redMenNum+((Box)enum1.nextElement()).getRedNum();
		}
		return redMenNum;
	}
	public long getBlueMen(){
		long blueMenNum = 0;
		Enumeration enum1 = bluePlayer.getOccupiedSquares().elements();
		while (enum1.hasMoreElements()){
			blueMenNum =  blueMenNum+((Box)enum1.nextElement()).getBlueNum();
		}
		return blueMenNum;
	}
	public int getRedSquares(){
		return redPlayer.getOccupiedSquares().size();
	}
	public int getBlueSquares(){
		return bluePlayer.getOccupiedSquares().size();
	}
	public long getRedPoints(){
		return getRedSquares()*squarePoints+getRedMen()*manPoints;
	}
	public long getBluePoints(){
		return getBlueSquares()*squarePoints+getBlueMen()*manPoints;
	}			
	public int getTurnsRemain(){
		return noTurns-turnsTaken;
	}
	public void setGameState(int gameStatePar){
		gameState = gameStatePar;
	}
	public Vector getSquaresVec(int colPar){
		if (colPar==Box.RED) return redPlayer.getSquaresVec();
		else return bluePlayer.getSquaresVec();
	}
	public void setSquare(int xPar, int yPar, int colPar){
		if (theGrid.getBox(xPar,yPar).getCol()==Box.EMPTY){
			if (colPar==Box.RED)theGrid.getBox(xPar,yPar).setCol(Box.RED);
			else if (colPar==Box.BLUE)theGrid.getBox(xPar,yPar).setCol(Box.BLUE);
		}
		else if (theGrid.getBox(xPar,yPar).getCol()==Box.RED){
			if (colPar==Box.BLUE)theGrid.getBox(xPar,yPar).setCol(Box.BOTH);
		}
		else if (theGrid.getBox(xPar,yPar).getCol()==Box.BLUE){
			if (colPar==Box.RED)theGrid.getBox(xPar,yPar).setCol(Box.BOTH);
		}
	}
	private boolean parseAddMenFunc(String theFunc){
		System.out.println("Entering Func 1");
		try{
			theFunc = subFunc(theFunc,2);
			theFunc = signExt(theFunc,2);
		}
		catch (Exception e){
			System.out.println(e.getMessage());
			return false;
		}
		Enumeration enum1 = tokens.elements();
		while (enum1.hasMoreElements()){
			System.out.print((String)enum1.nextElement()+" ");
		}
		System.out.println(theFunc);
		return theFunc.equals(""); 
	}
	private String subFunc(String theFunc, int num)throws ParseException{
		System.out.println("Entering subFunc "+num);
		if (theFunc.charAt(0)=='('){
			System.out.println("Eat '"+theFunc.charAt(0)+"'");
			curStr = curStr.concat(theFunc.substring(0,1));
			tokens.addElement(curStr);
			curStr = new String();
			theFunc = subFunc(theFunc.substring(1),num+1);
			theFunc = signExt(theFunc,num+1);
			if (theFunc.length()==0) throw new ParseException("Closing bracket expected.");
			if (theFunc.charAt(0)==')'){
				System.out.println("Eat '"+theFunc.charAt(0)+"'");
				curStr = curStr.concat(theFunc.substring(0,1));
				tokens.addElement(curStr);
				curStr = new String();
				theFunc = theFunc.substring(1);
			}
			else throw new ParseException("Closing bracket expected.");
		}
		else{
			theFunc = value(theFunc,num+1);
			theFunc = signExt(theFunc,num+1);
		}
		System.out.println("Exiting subFunc "+num);
		return theFunc;
	}
	private String subFuncExt(String theFunc, int num) throws ParseException{
		System.out.println("Entering subFuncExt "+num);
		if (theFunc.charAt(0)=='('){
			System.out.println("Eat '"+theFunc.charAt(0)+"'");
			curStr = curStr.concat(theFunc.substring(0,1));
			tokens.addElement(curStr);
			curStr = new String();
			theFunc = subFuncExt(theFunc.substring(1),num+1);
			theFunc = signExt(theFunc,num+1);
			if (theFunc.length()==0) throw new ParseException("Closing bracket expected.");
			if (theFunc.charAt(0)==')'){
				System.out.println("Eat '"+theFunc.charAt(0)+"'");
				curStr = curStr.concat(theFunc.substring(0,1));
				tokens.addElement(curStr);
				curStr = new String();
				theFunc = signExt(theFunc.substring(1),num+1);
			}
			else throw new ParseException("Closing bracket expected.");
		}
		else if ((theFunc.length()!=0)&&(theFunc.charAt(0)!=')')){ 
			theFunc = value(theFunc,num+1);
			theFunc = signExt(theFunc,num+1);
		}
		System.out.println("Exiting subFuncExt "+num);
		return theFunc;
	}
	private String signExt(String theFunc, int num) throws ParseException{
		System.out.println("Entering signExt "+num); 
		if (theFunc.length()!=0){
			if ((theFunc.charAt(0)=='+')||(theFunc.charAt(0)=='-')||(theFunc.charAt(0)=='*')||(theFunc.charAt(0)=='/')||(theFunc.charAt(0)=='^')){
				theFunc = sign(theFunc, num+1);
				theFunc = subFuncExt(theFunc, num+1);
			}
		}
		System.out.println("Exiting signExt "+num);
		return theFunc;
	}

	private String value(String theFunc, int num) throws ParseException{
		System.out.println("Entering value "+num);
		if (theFunc.charAt(0)=='X'){
			System.out.println("Eat '"+theFunc.charAt(0)+"'");
			curStr = curStr.concat(theFunc.substring(0,1));
			tokens.addElement(curStr);
			curStr = new String();
			theFunc = theFunc.substring(1);
		}
		else{ 
			theFunc = number(theFunc,num+1);
		}
		System.out.println("Exiting value "+num);
		return theFunc;
	}
	private String number(String theFunc, int num) throws ParseException{
		System.out.println("Entering number "+num);
		if (theFunc.charAt(0)=='-'){
			System.out.println("Eat '"+theFunc.charAt(0)+"'");
			curStr = curStr.concat(theFunc.substring(0,1));
			theFunc = theFunc.substring(1);
		}
		theFunc = preDecimalDigit(theFunc,num+1);
		System.out.println("Exiting number "+num);
		return theFunc;
	}

	private String preDecimalDigit(String theFunc, int num)throws ParseException{
		System.out.println("Entering preDecimalDigit "+num);
		boolean numFound = false;
		String digits = new String("0123456789");
		for (int i=0; i<digits.length(); i++){
			if (theFunc.charAt(0)==digits.charAt(i)){
				System.out.println("Eat '"+theFunc.charAt(0)+"'");
				curStr = curStr.concat(theFunc.substring(0,1));
				theFunc = preDecimalDigitExt(theFunc.substring(1),num+1);
				numFound = true;
				break;
			}
		}
		if (!numFound) throw new ParseException("Invalid Number");
		System.out.println("Exiting preDecimalDigit "+num);
		return theFunc;
	}

	private String preDecimalDigitExt(String theFunc, int num) throws ParseException{
		System.out.println("Entering preDecimalDigitExt "+num);
		boolean numFound = false;
		String digits = new String("0123456789");
		if (theFunc.length()!=0){
			for (int i=0; i<digits.length(); i++){
				if (theFunc.charAt(0)==digits.charAt(i)){
					System.out.println("Eat '"+theFunc.charAt(0)+"'");
					curStr = curStr.concat(theFunc.substring(0,1));
					theFunc = preDecimalDigitExt(theFunc.substring(1),num+1);
					numFound = true;
					break;
				}
			}
			if ((!numFound)&&(theFunc.charAt(0)=='.')){
				System.out.println("Eat '"+theFunc.charAt(0)+"'");
				curStr = curStr.concat(theFunc.substring(0,1));
				theFunc = postDecimalDigit(theFunc.substring(1),num+1);
			}
			else if (!numFound){
				tokens.addElement(curStr);
				curStr = new String();
			}
		}
		else{
			tokens.addElement(curStr);
			curStr = new String();
		}
		System.out.println("Exiting preDecimalDigitExt "+num);
		return theFunc;
	}
	private String postDecimalDigit(String theFunc, int num) throws ParseException{
		System.out.println("Entering postDecimalDigit "+num);
		boolean numFound = false;
		String digits = new String("0123456789");
		for (int i=0; i<digits.length(); i++){
			if (theFunc.charAt(0)==digits.charAt(i)){
				System.out.println("Eat '"+theFunc.charAt(0)+"'");
				curStr = curStr.concat(theFunc.substring(0,1));
				theFunc = postDecimalDigitExt(theFunc.substring(1),num+1);
				numFound = true;
				break;
			}
		}		 
		if (!numFound) throw new ParseException("Invalid Number");
		System.out.println("Exiting postDecimalDigit "+num);
		return theFunc;
	}
	private String postDecimalDigitExt(String theFunc, int num) throws ParseException{
		boolean numFound = false;
		System.out.println("Entering postDecimalDigitExt "+num);
		String digits = new String("0123456789");
		if (theFunc.length()!=0){
			for (int i=0; i<digits.length(); i++){
				if (theFunc.charAt(0)==digits.charAt(i)){
					numFound=true;
					System.out.println("Eat '"+theFunc.charAt(0)+"'");	
					curStr = curStr.concat(theFunc.substring(0,1));
					theFunc = postDecimalDigitExt(theFunc.substring(1),num+1);
					break;
				}
			}
		}
		if(!numFound){
			tokens.addElement(curStr);
			curStr = new String();
		}
		System.out.println("Exiting postDecimalDigitExt "+num);
		return theFunc;
	}
	private String sign(String theFunc, int num) throws ParseException{
		System.out.println("Entering sign "+num);
		if ((theFunc.charAt(0)=='+')||(theFunc.charAt(0)=='-')||(theFunc.charAt(0)=='*')||(theFunc.charAt(0)=='/')||(theFunc.charAt(0)=='^')){
			System.out.println("Eat '"+theFunc.charAt(0)+"'");
			curStr = curStr.concat(theFunc.substring(0,1));	
		}
		else throw new ParseException("Sign expected.");
		tokens.addElement(curStr);
		curStr = new String();
		System.out.println("Exiting sign "+num);
		return theFunc.substring(1);
	}
	private double calcNewNumMen(double numMen, Vector myTokens){
		for (int i=0; i<myTokens.size(); i++){
	//		System.out.println("Setting X's");
			if (((String)myTokens.elementAt(i)).equals("X"))
				myTokens.set(i,(new Double(numMen)).toString());
		}
		return calcNewNumMenRec(myTokens); 
	}

	private double calcNewNumMenRec(Vector myTokens){
//		System.out.println("Called calcNewNumMenRec");
		boolean openBracketsFound = true;
		boolean closeBracketsFound = true;
		int openBracket = 0;
		int closeBracket = 0;	
		while (openBracketsFound && closeBracketsFound){
			openBracketsFound=false;
			closeBracketsFound=false;
			for (int i=0; i<myTokens.size() && !closeBracketsFound; i++){
				if (((String)myTokens.elementAt(i)).equals("(")){
					openBracket=i;
					openBracketsFound=true;
				}
				if (((String)myTokens.elementAt(i)).equals(")")){	
					closeBracket=i;
					closeBracketsFound=true;
				}
			}
			if (closeBracketsFound==true){
				double res = calcNewNumMenRec(new Vector(myTokens.subList(openBracket+1, closeBracket)));
				for (int i=openBracket; i<=closeBracket; i++){
					 myTokens.removeElementAt(openBracket);
				}
				myTokens.insertElementAt((new Double(res)).toString(),openBracket);
			}			
		}
	//	System.out.println("Brackets finished");
		myTokens = calcSum(myTokens, '^');
	//	System.out.println("Powers finished");
		myTokens = calcSum(myTokens, '/');
	//	System.out.println("Divides finished");
		myTokens = calcSum(myTokens, '*');
	//	System.out.println("Multiplies finished");
		myTokens = calcSum(myTokens, '-');
	//	System.out.println("Subtracts finished");
		myTokens = calcSum(myTokens, '+');
	//	System.out.println("Adds finished");
		return new Double((String)myTokens.elementAt(0)).doubleValue();
	}
	private Vector calcSum(Vector myTokens, char signPar){
	//	System.out.println("CalcSum called");
		for (int y=0; y<myTokens.size(); y++){
	//		System.out.println("Checking My token "+y);
			if (signPar==((String)myTokens.elementAt(y)).charAt(0) && ((String)myTokens.elementAt(y)).length()==1){
			//	System.out.println("Checking Signs");
				if (signPar=='^') myTokens.insertElementAt(power((String)myTokens.elementAt(y-1),(String)myTokens.elementAt(y), (String)myTokens.elementAt(y+1)),y-1);
				else if (signPar =='/') myTokens.insertElementAt(divide((String)myTokens.elementAt(y-1),(String)myTokens.elementAt(y), (String)myTokens.elementAt(y+1)),y-1);
				else if (signPar =='*') myTokens.insertElementAt(multiply((String)myTokens.elementAt(y-1),(String)myTokens.elementAt(y), (String)myTokens.elementAt(y+1)),y-1);
				else if (signPar =='-') myTokens.insertElementAt(subtract((String)myTokens.elementAt(y-1),(String)myTokens.elementAt(y), (String)myTokens.elementAt(y+1)),y-1);					
				else if (signPar =='+') myTokens.insertElementAt(add((String)myTokens.elementAt(y-1),(String)myTokens.elementAt(y), (String)myTokens.elementAt(y+1)),y-1);
			//	System.out.println((String)myTokens.elementAt(y)+myTokens.elementAt(y+1)+myTokens.elementAt(y+2)+"="+myTokens.elementAt(y-1));
				myTokens.removeElementAt(y);
				myTokens.removeElementAt(y);
				myTokens.removeElementAt(y);
				y=0;	
			}
		}
		return myTokens;
	}		 
		
	private String power(String num, String sign, String power){
	//	System.out.println("Power "+num+" "+sign+" "+power);
		double num1 = (new Double(num)).doubleValue();
		double num2 = (new Double(power)).doubleValue();
		double result = java.lang.Math.pow(num1,num2);
		return new Double(result).toString();
	}
	private String divide(String dividend, String sign, String divisor){
	//	System.out.println("Divide "+dividend+" "+sign+" "+divisor);
		double num1 = (new Double(dividend)).doubleValue();
		double num2 = (new Double(divisor)).doubleValue();
		double result = num1/num2;
		return new Double(result).toString();
	}
	private String multiply(String num, String sign, String multiplier){
	//	System.out.println("Multiply "+num+" "+sign+" "+multiplier);
		double num1 = (new Double(num)).doubleValue();
		double num2 = (new Double(multiplier)).doubleValue();
		double result = num1*num2;
		return new Double(result).toString();
	}
	private String subtract(String num, String sign, String subtracter){
	//	System.out.println("Subtract "+num+" "+sign+" "+subtracter);
		double num1 = (new Double(num)).doubleValue();
		double num2 = (new Double(subtracter)).doubleValue();
		double result = num1-num2;
		return new Double(result).toString();
	}
	private String add(String num, String sign, String adder){
	//	System.out.println("Add "+num+" "+sign+" "+adder);
		double num1 = (new Double(num)).doubleValue();
		double num2 = (new Double(adder)).doubleValue();
		double result = num1+num2;
		return new Double(result).toString();
	}

	public Vector getWithdraws(Box wdBox, int colPar){
		Vector allWDs = new Vector();
		try{
		int wdHoriz = wdBox.getHorNum();
		int wdVert = wdBox.getVertNum();
		int avoidCol;
		
		if (colPar==Box.RED) avoidCol=Box.BLUE;
		else avoidCol=Box.RED;
		if (wdHoriz>0){
			if (theGrid.getBox(wdHoriz-1,wdVert).getNum(avoidCol)==0) allWDs.addElement(new RiskGameMove(wdBox.getBoxPosition(), new BoxPosition(wdHoriz-1,wdVert)));
		}
		if (wdHoriz<gridX-1){
			if (theGrid.getBox(wdHoriz+1,wdVert).getNum(avoidCol)==0) allWDs.addElement(new RiskGameMove(wdBox.getBoxPosition(), new BoxPosition(wdHoriz+1,wdVert)));
		}
		if (wdVert>0){
			if (theGrid.getBox(wdHoriz,wdVert-1).getNum(avoidCol)==0) allWDs.addElement(new RiskGameMove(wdBox.getBoxPosition(), new BoxPosition(wdHoriz,wdVert-1)));
		}
		if (wdVert<gridY-1){
			if (theGrid.getBox(wdHoriz,wdVert+1).getNum(avoidCol)==0) allWDs.addElement(new RiskGameMove(wdBox.getBoxPosition(), new BoxPosition(wdHoriz,wdVert+1)));	
		}
		if (diagonalSquaresAdjacent){
			if (wdHoriz>0 && wdVert>0){
				if (theGrid.getBox(wdHoriz-1,wdVert-1).getNum(avoidCol)==0) allWDs.addElement(new RiskGameMove(wdBox.getBoxPosition(), new BoxPosition(wdHoriz-1,wdVert-1)));
			}
			if (wdHoriz<gridX-1 && wdVert<gridY-1){
				if (theGrid.getBox(wdHoriz+1,wdVert+1).getNum(avoidCol)==0) allWDs.addElement(new RiskGameMove(wdBox.getBoxPosition(), new BoxPosition(wdHoriz+1,wdVert+1)));
			}
			if (wdHoriz<gridX-1 && wdVert>0){
				if (theGrid.getBox(wdHoriz+1,wdVert-1).getNum(avoidCol)==0) allWDs.addElement(new RiskGameMove(wdBox.getBoxPosition(), new BoxPosition(wdHoriz+1,wdVert-1)));
			}
			if (wdHoriz>0 && wdVert<gridY-1){
				if (theGrid.getBox(wdHoriz-1,wdVert+1).getNum(avoidCol)==0) allWDs.addElement(new RiskGameMove(wdBox.getBoxPosition(), new BoxPosition(wdHoriz-1,wdVert+1)));	
			}
		}
		}catch (Exception e){System.out.println("Error in RiskGame.getWithdraws method");} 
		return allWDs;
	}
		
	public static void main(String[] argv){
		RiskGame theGame = new RiskGame();
		System.out.println(argv[0]);
		if (theGame.parseAddMenFunc(argv[0])){
			double result = theGame.calcNewNumMen(25, theGame.tokens);
			System.out.println(argv[0]+" = "+result);
		}
	}
}
		



		
